==> viber_channel-1.0.0.tar.gz: viber_channel/src/Form/ViberChannelSendForm.php <==
<?php

namespace Drupal\viber_channel\Form;

use Drupal\Component\Utility\Html;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Markup;
use Drupal\Core\Url;
use Drupal\file\Entity\File;
use Drupal\viber_channel\API\IContentRequestManager;
use Drupal\viber_channel\API\ViberApi;
use Drupal\viber_channel\Http\HttpClient;
use Drupal\viber_channel\Model\ViberChannelServiceInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\viber_channel\Query\ViberChannelQuery;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Cache\CacheableDependencyInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;

/**
 *
 */
class ViberChannelSendForm extends FormBase {

  /**
   * Folder for save temporality photo which will be sent.
   */
  const FOLDER = 'public://viber/channel_files/';

  /**
   * Type of message.
   */
  const DEF_TYPE = 'text';

  /**
   * Available tags for send to viber.
   */
  const TAGS = '';

  /**
   * @var ViberChannelServiceInterface|null
   */
  protected ?ViberChannelServiceInterface $viberService = NULL;

  /**
   * @var \Drupal\viber_channel\Query\ViberChannelQuery|null
   */
  private ?ViberChannelQuery $viberQuery = NULL;

  /**
   * Provides an interface for entity type managers.
   *
   * @var EntityTypeManagerInterface|NULL
   */
  private ?EntityTypeManagerInterface $entityTypeManager;

  /**
   * Provides an interface for an entity repository.
   *
   * @var \Drupal\Core\Entity\EntityRepositoryInterface|null
   */
  private ?EntityRepositoryInterface $entityRepository = NULL;

  /**
   * Defines an interface for objects which may be used by other cached objects.
   *
   * @var \Drupal\Core\Cache\CacheableDependencyInterface|null
   */
  private ?CacheableDependencyInterface $viberConfig = NULL;

  /**
   * Request represents an HTTP request.
   *
   * @var \Symfony\Component\HttpFoundation\Request|null
   */
  private ?Request $request = NULL;

  /**
   * General interface for coordinating the receipt of content from a remote
   * api.
   *
   * @var \Drupal\viber_channel\API\IContentRequestManager|null
   */
  private ?IContentRequestManager $viberRequest = NULL;

  private ?RendererInterface $renderer = NULL;

  private ?FileUrlGeneratorInterface $fileUrlGenerator = NULL;

  private ?ModuleHandlerInterface $moduleHandler = NULL;

  public function __construct() {
    $this->initProperties();
  }

  public function initProperties() {
    $container = \Drupal::getContainer();
    $this->viberService = $container->get('viber_channel');
    $this->viberQuery = $container->get('viber_channel.query');
    $this->entityTypeManager = $container->get('entity_type.manager');
    $this->entityRepository = $container->get('entity.repository');
    $this->viberConfig = $container->get('config.factory')
      ->get(ViberChannelSettingsForm::CONFIG_NAME);
    $this->request = $container->get('request_stack')->getCurrentRequest();
    $this->viberRequest = $container->get('viber_channel.request');
    $this->renderer = $container->get('renderer');
    $this->fileUrlGenerator = $container->get('file_url_generator');
    $this->moduleHandler = $container->get('module_handler');
  }

  /**
   * {@inheritDoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form_id = $this->getFormId();

    if (!$this->viberQuery) {
      $this->initProperties();
    }

    $listOfChannelUsers = $this->viberQuery->getUsers();
    if (!$listOfChannelUsers) {
      return ['#markup' => $this->t('Users not found')];
    }

    $storage = $form_state->getStorage();
    if (empty($form_state->getValues())) {
      $form_state->setValue('type', $this::DEF_TYPE);
    }
    if (empty($form_state->getValue('type')) && !empty($storage['type'])) {
      $form_state->setValue('type', $storage['type']);
    }
    $storage['type'] = $form_state->getValue('type');
    $form_state->setStorage($storage);

    $form['#prefix'] = "<div id='$form_id'>";
    $form['#suffix'] = '</div>';

    $tokens = array_column($listOfChannelUsers, 'sid');
    $names = array_column($listOfChannelUsers, 'user_name');
    $options = array_combine($tokens, $names);

    $form['sender'] = [
      '#type' => 'select',
      '#title' => $this->t('Choose sender'),
      '#description' => $this->t('On whose behalf the messages will be sent'),
      '#options' => $options,
      '#required' => TRUE,
    ];

    $form['type'] = [
      '#type' => 'select',
      '#title' => $this->t('Message type'),
      '#required' => TRUE,
      '#options' => [
        'text' => $this->t('Message'),
        'photo' => $this->t('Photo with caption'),
      ],
      '#default_value' => $form_state->getValue('type'),
      '#ajax' => [
        'wrapper' => $form_id,
        'callback' => [$this, 'ajaxCallback'],
        'event' => 'change',
      ],
    ];

    $messageInfo = $this->getMessageValue($form_state->getValue('type'), $form_state);
    switch ($form_state->getValue('type')) {
      case'text':
        $form['message'] = [
          '#type' => 'text_format',
          '#title' => $this->t('Message'),
          '#format' => 'viber_html',
          '#after_build' => [[$this, 'textFormatHide']],
          '#default_value' => !empty($messageInfo['message']) ? $messageInfo['message'] : '',
          '#required' => TRUE,
          '#description' => $this->t('The signature should not contain links and be more than 7000 characters.'),
        ];
        break;
      case'photo':
        $form['file'] = [
          '#type' => 'managed_file',
          '#title' => t('File'),
          '#upload_location' => $this::FOLDER,
          '#required' => TRUE,
          '#upload_validators' => [
            'file_validate_extensions' => ['png gif jpg jpeg'],
          ],
          '#default_value' => !empty($messageInfo['photo']['fid']) ? [$messageInfo['photo']['fid']] : NULL,
        ];
        $form['caption'] = [
          '#type' => 'text_format',
          '#title' => $this->t('Caption'),
          '#format' => 'viber_html',
          '#after_build' => [[$this, 'textFormatHide']],
          '#default_value' => !empty($messageInfo['photo']['caption']) ? $messageInfo['photo']['caption'] : '',
          '#description' => $this->t('The signature should not contain links and be more than 768 characters.'),
        ];
        break;
    }

    $form['actions'] = [
      '#type' => 'actions',
      '#weight' => 100,
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Send'),
        '#name' => 'send',
      ],
    ];
    return $form;
  }

  /**
   * {@inheritDoc}
   */
  public function getFormId(): string {
    return 'viber_channel_send_form';
  }

  /**
   * @param $type
   * @param $form_state
   *
   * @return array
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function getMessageValue($type, $form_state): array {
    $items = [];
    if (!empty($nid = $this->request->query->get('send_nid'))) {
      $entity = $this->entityTypeManager->getStorage('node')->load($nid);
      if (!empty($entity)) {
        /** @var \Drupal\node\Entity\Node $entity */
        $entity = $this->entityRepository->getTranslationFromContext($entity);
        $items[] = $entity->getTitle();
        $items[] = Url::fromRoute('entity.node.canonical', ['node' => $entity->id()], ['absolute' => TRUE])
          ->toString();
      }
      $form_state->set('sendNid', $nid);
    }
    $message = implode(PHP_EOL, $items);
    $messageInfo = [
      'message' => $message,
      'photo' => [
        'fid' => 0,
        'caption' => $message,
      ],
    ];

    /** Create alter for message */
    $this->moduleHandler->alter('viber_channel_message', $messageInfo, $type);

    return $messageInfo;
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return array
   */
  public function ajaxCallback(array $form, FormStateInterface $form_state): array {
    return $form;
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return void
   * @throws \DOMException
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \GuzzleHttp\Exception\GuzzleException
   * @throws \Exception
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    if (!$this->viberQuery) {
      $this->initProperties();
    }
    if (!empty($form_state->get('sendRun'))) {
      return;
    }

    $sender = $form_state->getValue('sender');
    $token = \Drupal::config(ViberChannelSettingsForm::CONFIG_NAME)
      ->get('config')['auth_token'];

    $triggerElement = $form_state->getTriggeringElement();
    $client = (new HttpClient())->setEndpointUri(ViberApi::POST)
      ->withBody('auth_token', $token)
      ->setMethod('POST');

    if (!empty($triggerElement['#name']) && $triggerElement['#name'] == 'send') {

      switch ($form_state->getValue('type')) {
        case'text':
          // Replace img by attach
          $html = Html::load($form_state->getValue(['message', 'value']));
          $replaces = [];
          foreach ($html->getElementsByTagName('img') as $img) {
            $element = $html->createElement('a', '&#8205;');
            $element->setAttribute('href', $img->getAttribute('src'));
            $replaces[] = [
              'old' => $img,
              'new' => $element,
            ];
          }
          $this->replaceChild($replaces);
          $message = Html::serialize($html);
          $message = [
            '#type' => 'processed_text',
            '#text' => $message,
            '#format' => $form_state->getValue(['message', 'format']),
          ];
          $message = $this->renderer->render($message);
          $message = str_replace([
            'href="/',
            '&nbsp;',
            '<p></p>',
          ], [
            'href="' . $GLOBALS['base_url'] . '/',
            '',
            '',
          ], $message);
          // ---
          $message = strip_tags($message, $this::TAGS);
          $message = substr($message, 0, 7000);
          $client->withBody('text', $message)
            ->withBody('type', 'text')
            ->withBody('from', $sender);
          $result = $this->viberRequest->send($client);
          if (!empty($result) && $result['status'] == 0) {
            $this->messenger()
              ->addMessage($this->t('Message sent successfully.'), 'status', FALSE);
          }
          else {
            $form_state->setErrorByName('message][value', $this->t('Error: message send failed.'));
          }
          $this->sendLog($form_state, [$message]);
          break;

        case'photo':
          $message = [
            '#type' => 'processed_text',
            '#text' => $form_state->getValue(['caption', 'value']),
            '#format' => $form_state->getValue(['caption', 'format']),
          ];
          $message = $this->renderer->render($message);
          // ---
          $message = strip_tags($message, $this::TAGS);
          $photo = '';
          $file = $this->entityTypeManager->getStorage('file')
            ->load($form_state->getValue(['file', 0]));
          if (!empty($file)) {
            $url = $this->fileUrlGenerator->generateAbsoluteString($file->getFileUri());
            $photo = Url::fromUri($url)->toString();
          }

          $message = substr($message, 0, 768);
          $client->withBody('from', $sender)
            ->withBody('type', 'picture')
            ->withBody('media', $photo)
            ->withBody('text', $message);
          $result = $this->viberRequest->send($client);

          if (!empty($result) && $result['status'] == 0) {
            $this->messenger()
              ->addMessage($this->t('Message sent successfully.'), 'status', FALSE);
          }
          else {
            $form_state->setErrorByName('caption][value', $this->t('Error: message send failed.'));
          }
          $this->sendLog($form_state, [$message]);
          break;
      }

    }
  }

  /**
   * @param $replaces
   *
   * @return void
   */
  private function replaceChild($replaces) {
    if (!empty($replaces)) {
      foreach ($replaces as $replace) {
        $replace['old']->parentNode->replaceChild($replace['new'], $replace['old']);
      }
    }
  }

  /**
   * Write log about sent messages.
   *
   * @param FormStateInterface $form_state
   * @param array $data
   *
   * @return void
   */
  public function sendLog(FormStateInterface $form_state, array $data = []) {
    $form_state->set('sendRun', TRUE);
    if (!empty($form_state->get('sendNid'))) {
      \Drupal::service('viber_channel.query')->writeLog([
        'nid' => $form_state->get('sendNid'),
        'user' => \Drupal::currentUser()->getDisplayName(),
      ], $data);
    }
  }

  public static function deleteFiles() {
    $fids = \Drupal::database()
      ->select('file_managed', 'f')
      ->fields('f', ['fid'])
      ->condition('f.uri', self::FOLDER . '%', 'LIKE')
      ->execute()
      ->fetchCol();
    if (!empty($fids)) {
      foreach (File::loadMultiple($fids) as $file) {
        $file->delete();
      }
    }
  }

  /**
   * @param $element
   *
   * @return array
   */
  public function textFormatHide($element): array {
    $element['format']['#prefix'] = Markup::create('<div style="display:none;">');
    $element['format']['#suffix'] = '</div>';
    return $element;
  }

  /**
   * {@inheritDoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // TODO: Implement submitForm() method.
  }

}

==> viber_channel-1.0.1.tar.gz: viber_channel/src/Form/ViberChannelSendForm.php <==
<?php

namespace Drupal\viber_channel\Form;

use Drupal\Component\Utility\Html;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Markup;
use Drupal\Core\Url;
use Drupal\file\Entity\File;
use Drupal\viber_channel\API\IContentRequestManager;
use Drupal\viber_channel\API\ViberApi;
use Drupal\viber_channel\Http\HttpClient;
use Drupal\viber_channel\Model\ViberChannelServiceInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\viber_channel\Query\ViberChannelQuery;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Cache\CacheableDependencyInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;

/**
 *
 */
class ViberChannelSendForm extends FormBase {

  /**
   * Folder for save temporality photo which will be sent.
   */
  const FOLDER = 'public://viber/channel_files/';

  /**
   * Type of message.
   */
  const DEF_TYPE = 'text';

  /**
   * Available tags for send to viber.
   */
  const TAGS = '';

  /**
   * @var ViberChannelServiceInterface|null
   */
  protected $viberService = NULL;

  /**
   * @var \Drupal\viber_channel\Query\ViberChannelQuery|null
   */
  private $viberQuery = NULL;

  /**
   * Provides an interface for entity type managers.
   *
   * @var EntityTypeManagerInterface|NULL
   */
  private $entityTypeManager;

  /**
   * Provides an interface for an entity repository.
   *
   * @var \Drupal\Core\Entity\EntityRepositoryInterface|null
   */
  private $entityRepository = NULL;

  /**
   * Defines an interface for objects which may be used by other cached objects.
   *
   * @var \Drupal\Core\Cache\CacheableDependencyInterface|null
   */
  private $viberConfig = NULL;

  /**
   * Request represents an HTTP request.
   *
   * @var \Symfony\Component\HttpFoundation\Request|null
   */
  private $request = NULL;

  /**
   * General interface for coordinating the receipt of content from a remote
   * api.
   *
   * @var \Drupal\viber_channel\API\IContentRequestManager|null
   */
  private $viberRequest = NULL;

  /**
   * @var null
   */
  private $renderer = NULL;

  /**
   * @var null
   */
  private $fileUrlGenerator = NULL;

  /**
   * @var null
   */
  private $moduleHandler = NULL;

  /**
   *
   */
  public function __construct() {
    $this->initProperties();
  }

  /**
   * @return void
   */
  public function initProperties() {
    $container = \Drupal::getContainer();
    $this->viberService = $container->get('viber_channel');
    $this->viberQuery = $container->get('viber_channel.query');
    $this->entityTypeManager = $container->get('entity_type.manager');
    $this->entityRepository = $container->get('entity.repository');
    $this->viberConfig = $container->get('config.factory')
      ->get(ViberChannelSettingsForm::CONFIG_NAME);
    $this->request = $container->get('request_stack')->getCurrentRequest();
    $this->viberRequest = $container->get('viber_channel.request');
    $this->renderer = $container->get('renderer');
    $this->fileUrlGenerator = $container->get('file_url_generator');
    $this->moduleHandler = $container->get('module_handler');
  }

  /**
   * {@inheritDoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form_id = $this->getFormId();

    if (!$this->viberQuery) {
      $this->initProperties();
    }

    $listOfChannelUsers = $this->viberQuery->getUsers();
    if (!$listOfChannelUsers) {
      return ['#markup' => $this->t('Users not found')];
    }

    $storage = $form_state->getStorage();
    if (empty($form_state->getValues())) {
      $form_state->setValue('type', $this::DEF_TYPE);
    }
    if (empty($form_state->getValue('type')) && !empty($storage['type'])) {
      $form_state->setValue('type', $storage['type']);
    }
    $storage['type'] = $form_state->getValue('type');
    $form_state->setStorage($storage);

    $form['#prefix'] = "<div id='$form_id'>";
    $form['#suffix'] = '</div>';

    $tokens = array_column($listOfChannelUsers, 'sid');
    $names = array_column($listOfChannelUsers, 'user_name');
    $options = array_combine($tokens, $names);

    $form['sender'] = [
      '#type' => 'select',
      '#title' => $this->t('Choose sender'),
      '#description' => $this->t('On whose behalf the messages will be sent'),
      '#options' => $options,
      '#required' => TRUE,
    ];

    $form['type'] = [
      '#type' => 'select',
      '#title' => $this->t('Message type'),
      '#required' => TRUE,
      '#options' => [
        'text' => $this->t('Message'),
        'photo' => $this->t('Photo with caption'),
      ],
      '#default_value' => $form_state->getValue('type'),
      '#ajax' => [
        'wrapper' => $form_id,
        'callback' => [$this, 'ajaxCallback'],
        'event' => 'change',
      ],
    ];

    $messageInfo = $this->getMessageValue($form_state->getValue('type'), $form_state);
    switch ($form_state->getValue('type')) {
      case'text':
        $form['message'] = [
          '#type' => 'text_format',
          '#title' => $this->t('Message'),
          '#format' => 'viber_html',
          '#after_build' => [[$this, 'textFormatHide']],
          '#default_value' => !empty($messageInfo['message']) ? $messageInfo['message'] : '',
          '#required' => TRUE,
          '#description' => $this->t('The signature should not contain links and be more than 7000 characters.'),
        ];
        break;
      case'photo':
        $form['file'] = [
          '#type' => 'managed_file',
          '#title' => t('File'),
          '#upload_location' => $this::FOLDER,
          '#required' => TRUE,
          '#upload_validators' => [
            'file_validate_extensions' => ['png gif jpg jpeg'],
          ],
          '#default_value' => !empty($messageInfo['photo']['fid']) ? [$messageInfo['photo']['fid']] : NULL,
        ];
        $form['caption'] = [
          '#type' => 'text_format',
          '#title' => $this->t('Caption'),
          '#format' => 'viber_html',
          '#after_build' => [[$this, 'textFormatHide']],
          '#default_value' => !empty($messageInfo['photo']['caption']) ? $messageInfo['photo']['caption'] : '',
          '#description' => $this->t('The signature should not contain links and be more than 768 characters.'),
        ];
        break;
    }

    $form['actions'] = [
      '#type' => 'actions',
      '#weight' => 100,
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Send'),
        '#name' => 'send',
      ],
    ];
    return $form;
  }

  /**
   * {@inheritDoc}
   */
  public function getFormId(): string {
    return 'viber_channel_send_form';
  }

  /**
   * @param $type
   * @param $form_state
   *
   * @return array
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function getMessageValue($type, $form_state): array {
    $items = [];
    if (!empty($nid = $this->request->query->get('send_nid'))) {
      $entity = $this->entityTypeManager->getStorage('node')->load($nid);
      if (!empty($entity)) {
        /** @var \Drupal\node\Entity\Node $entity */
        $entity = $this->entityRepository->getTranslationFromContext($entity);
        $items[] = $entity->getTitle();
        $items[] = Url::fromRoute('entity.node.canonical', ['node' => $entity->id()], ['absolute' => TRUE])
          ->toString();
      }
      $form_state->set('sendNid', $nid);
    }
    $message = implode(PHP_EOL, $items);
    $messageInfo = [
      'message' => $message,
      'photo' => [
        'fid' => 0,
        'caption' => $message,
      ],
    ];

    /** Create alter for message */
    $this->moduleHandler->alter('viber_channel_message', $messageInfo, $type);

    return $messageInfo;
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return array
   */
  public function ajaxCallback(array $form, FormStateInterface $form_state): array {
    return $form;
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return void
   * @throws \DOMException
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \GuzzleHttp\Exception\GuzzleException
   * @throws \Exception
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    if (!$this->viberQuery) {
      $this->initProperties();
    }
    if (!empty($form_state->get('sendRun'))) {
      return;
    }

    $sender = $form_state->getValue('sender');
    $token = \Drupal::config(ViberChannelSettingsForm::CONFIG_NAME)
      ->get('config')['auth_token'];

    $triggerElement = $form_state->getTriggeringElement();
    $client = (new HttpClient())->setEndpointUri(ViberApi::POST)
      ->withBody('auth_token', $token)
      ->setMethod('POST');

    if (!empty($triggerElement['#name']) && $triggerElement['#name'] == 'send') {

      switch ($form_state->getValue('type')) {
        case'text':
          // Replace img by attach
          $html = Html::load($form_state->getValue(['message', 'value']));
          $replaces = [];
          foreach ($html->getElementsByTagName('img') as $img) {
            $element = $html->createElement('a', '&#8205;');
            $element->setAttribute('href', $img->getAttribute('src'));
            $replaces[] = [
              'old' => $img,
              'new' => $element,
            ];
          }
          $this->replaceChild($replaces);
          $message = Html::serialize($html);
          $message = [
            '#type' => 'processed_text',
            '#text' => $message,
            '#format' => $form_state->getValue(['message', 'format']),
          ];
          $message = $this->renderer->render($message);
          $message = str_replace([
            'href="/',
            '&nbsp;',
            '<p></p>',
          ], [
            'href="' . $GLOBALS['base_url'] . '/',
            '',
            '',
          ], $message);
          // ---
          $message = substr($message, 0, 7000);
          $message = strip_tags($message, $this::TAGS);
          $message = str_replace('&#13', '', $message);
          $client->withBody('text', $message)
            ->withBody('type', 'text')
            ->withBody('from', $sender);
          $result = $this->viberRequest->send($client);
          if (!empty($result) && $result['status'] == 0) {
            $this->messenger()
              ->addMessage($this->t('Message sent successfully.'), 'status', FALSE);
          }
          else {
            $form_state->setErrorByName('message][value', $this->t('Error: message send failed.'));
          }
          $this->sendLog($form_state, [$message]);
          break;

        case'photo':
          $message = [
            '#type' => 'processed_text',
            '#text' => $form_state->getValue(['caption', 'value']),
            '#format' => $form_state->getValue(['caption', 'format']),
          ];
          $message = $this->renderer->render($message);
          // ---
          $message = strip_tags($message, $this::TAGS);
          $photo = '';
          $file = $this->entityTypeManager->getStorage('file')
            ->load($form_state->getValue(['file', 0]));
          if (!empty($file)) {
            $url = $this->fileUrlGenerator->generateAbsoluteString($file->getFileUri());
            $photo = Url::fromUri($url)->toString();
          }

          $message = substr($message, 0, 768);
          $message = str_replace('&#13', '', $message);
          $client->withBody('from', $sender)
            ->withBody('type', 'picture')
            ->withBody('media', $photo)
            ->withBody('text', $message);
          $result = $this->viberRequest->send($client);

          if (!empty($result) && $result['status'] == 0) {
            $this->messenger()
              ->addMessage($this->t('Message sent successfully.'), 'status', FALSE);
          }
          else {
            $form_state->setErrorByName('caption][value', $this->t('Error: message send failed.'));
          }
          $this->sendLog($form_state, [$message]);
          break;
      }

    }
  }

  /**
   * @param $replaces
   *
   * @return void
   */
  private function replaceChild($replaces) {
    if (!empty($replaces)) {
      foreach ($replaces as $replace) {
        $replace['old']->parentNode->replaceChild($replace['new'], $replace['old']);
      }
    }
  }

  /**
   * Write log about sent messages.
   *
   * @param FormStateInterface $form_state
   * @param array $data
   *
   * @return void
   */
  public function sendLog(FormStateInterface $form_state, array $data = []) {
    $form_state->set('sendRun', TRUE);
    if (!empty($form_state->get('sendNid'))) {
      \Drupal::service('viber_channel.query')->writeLog([
        'nid' => $form_state->get('sendNid'),
        'user' => \Drupal::currentUser()->getDisplayName(),
      ], $data);
    }
  }

  /**
   * @return void
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public static function deleteFiles() {
    $fids = \Drupal::database()
      ->select('file_managed', 'f')
      ->fields('f', ['fid'])
      ->condition('f.uri', self::FOLDER . '%', 'LIKE')
      ->execute()
      ->fetchCol();
    if (!empty($fids)) {
      foreach (File::loadMultiple($fids) as $file) {
        $file->delete();
      }
    }
  }

  /**
   * @param $element
   *
   * @return array
   */
  public function textFormatHide($element): array {
    $element['format']['#prefix'] = Markup::create('<div style="display:none;">');
    $element['format']['#suffix'] = '</div>';
    return $element;
  }

  /**
   * {@inheritDoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // TODO: Implement submitForm() method.
  }

}
